#ifdef USE_GUI

#ifndef GUIH
#define GUIH

#include "console.h"


/*=========*/
/* Windows */
/*=========*/

#define BGCOL       CON_BLUE
#define TXTCOL      CON_BRIGHTWHITE
#define SHORTCUTCOL CON_LIGHTGREEN
#define REVERSECOL  CON_WHITE
#define BORDERCOL   CON_BRIGHTWHITE

#define TOPLEFT  0x1
#define TOPRIGHT 0x2
#define BOTLEFT  0x4
#define BOTRIGHT 0x8

struct gui_window {
  UBY x1, y1, x2, y2;
  UBY fgcolor;
  UBY bgcolor;
  UBY shortcutcolor;
  UBY reversecolor;
  UBY cornerflags;
};

void guiWindowClear(struct gui_window *w, BOOLE borderclear);
void guiWindowBorderPlot(struct gui_window *w);
void guiWindowTextPlot(struct gui_window *w, STR *s, ULO x, ULO y);
void guiWindowTextPlotReverse(struct gui_window *w, STR *s, ULO x, ULO y);
void guiWindowTextPlotShortcut(struct gui_window *w, STR *s, ULO x, ULO y,
			       ULO shortcutpos);
void guiWindowTextPlotReverseShortcut(struct gui_window *w, STR *s, ULO x,
				      ULO y, ULO shortcutpos);
void guiWindowInit(struct gui_window *w, UBY x1, UBY x2, UBY y1, UBY y2,
		   UBY bgcolor, UBY fgcolor, UBY shortcutcolor,
		   UBY reversecolor, UBY cornerflags);
void guiSpacesToEOL(struct gui_window *w, STR *s, ULO y);
void guiScreenSetup(void);
void guiScreenClose(void);


/*=======*/
/* Menus */
/*=======*/

#define GUI_MENU_LEVEL_UP 9999

struct gui_menu {
  STR **entrytext;
  ULO activeentry;
  struct gui_window *w;
  UBY *shortcuts;
};

void guiMenuInit(struct gui_menu *m, STR *entries[], ULO activeentry,
		 struct gui_window *w, UBY *shortcuts);
ULO guiMenuSelect(struct gui_menu *m, BOOLE toplevel);
void guiMenuPrint(struct gui_menu *m);
ULO guiMenuEntryWidth(struct gui_menu *m);
ULO guiMenuEntryCount(struct gui_menu *m);


/*============*/
/* Optionmenu */
/*============*/

#define GUI_FILEREQ_ALL 0
#define GUI_VALUELIST   1
#define GUI_FILEREQ_ROM 2
#define GUI_FILEREQ_ADF 3
#define GUI_FILEREQ_HDF 4
#define GUI_FILEREQ_KEY 5

#define GUI_OPTIONMENU_DEC 0
#define GUI_OPTIONMENU_INC 1

struct gui_optionmenu {
  STR *topheader;
  STR **headers;
  UBY *actions;
  void *vars;
  STR ***texts;
  void **vals;
  struct gui_window *w;
  UBY *shortcuts;
};

void guiOptionmenuInit(struct gui_optionmenu *om, STR *topheader,
		       STR **headers, UBY *actions, void *vars, STR ***texts,
		       void **vals, struct gui_window *w, UBY *shortcuts);
void guiOptionmenuRun(struct gui_optionmenu *om);
void guiMemnamesClear(void);


/*========*/
/* Fields */
/*========*/

struct gui_numberfield {
  ULO x,y;      /* Start position in window */
  ULO value;
  struct gui_window *hostwindow;
  ULO hex;
};

struct gui_textfield {
  ULO x, y;      /* Start position in window */
  STR value[80];
  struct gui_window *hostwindow;
  ULO maxlen;
};

BOOLE guiFieldNumberEnter(struct gui_numberfield *n);
void guiFieldNumberInit(struct gui_numberfield *n, struct gui_window *w, ULO v,
			ULO x, ULO y, ULO hex);
BOOLE guiFieldTextEnter(struct gui_textfield *n);
void guiFieldTextInit(struct gui_textfield *n, struct gui_window *w, STR *s,
		      ULO x, ULO y, ULO maxlen);


/*==========================*/
/* Other data used in gui.c */
/*==========================*/

extern BOOLE gui_exitflag;
extern STR config_diskpath[];
extern ULO config_altn_loop;
extern STR menupos_act[];
extern STR menupos_want0[];
extern STR menupos_want1[];
extern ULO config_store_mpos;
extern ULO config_autorun;
extern STR memnames[10][256];

#endif /* GUIH */
#endif /* USE_GUI */
