/*--------------------------------------------*/
/* UAE to Fellow bridge header file           */
/* In the case where similar functions exist, */
/* macros are defined which expand to the     */
/* Fellow equivalent.                         */
/*--------------------------------------------*/

#ifndef UAE2FELLH
#define UAE2FELLH

#include "defs.h"
#include "68000.h"
#include "fhfile.h"
#include "memory.h"


/*------------------------*/
/* Base types             */
/*------------------------*/

#define ULONG unsigned int
#define uae_u32 unsigned int
#define uae_s32 int
#define uae_u16 unsigned short
#define uae_s16 short
#define uae_u8 unsigned char
#define uae_s8 signed char
#define uaecptr unsigned int

#define __DOS__

/*------------------------*/
/* Misc types, funcs      */
/*------------------------*/

#define my_strdup strdup
#define strcasecmp stricmp
#define mode_t unsigned int
#define my_mkdir(P, M) (mkdir(P))
#define REGPARAM2
#define REGPARAM
#define version 31
#define uae_quit fellow_nasty_exit

/*================*/
/* UAE's Hardfile */
/*================*/

extern char uaehf0[256];
extern ULO hardfile_size;

/*------------------------*/
/* Access m68k registers  */
/*------------------------*/

#define m68k_dreg(REGS, NR) (d[NR])
#define m68k_areg(REGS, NR) (a[NR])
#define m68k_setpc(I) (set_pc(I))
extern void m68k_do_rts(void);

/*------------------------*/
/* Memory interfacing     */
/*------------------------*/

#define get_long(ADR) (fetl(ADR))
#define get_word(ADR) (fetw(ADR))
#define get_byte(ADR) (fetb(ADR))

#define put_long(ADR, DATA) (wril(DATA, ADR))
#define put_word(ADR, DATA) (wriw(DATA, ADR))
#define put_byte(ADR, DATA) (wrib(DATA, ADR))

#define expamem_write(OFFSET, DATA) (memoryEmemSet(OFFSET,DATA))
#define expamem emem
#define expamem_init_clear memoryEmemClear
#define chipmem_size config_memory_chipsize
extern uae_u8 filesysory[65536];

/*===========================================*/
/* These access memory with a direct pointer */
/*===========================================*/

extern ULO do_get_mem_byte(uae_u8 *address);
extern ULO do_get_mem_word(uae_u16 *address);
extern ULO do_get_mem_long(uae_u32 *address);
extern void do_put_mem_long(uae_u32 *address, uae_u32 data);
extern void do_put_mem_word(uae_u16 *address, uae_u16 data);
extern void do_put_mem_byte(uae_u8 *address, uae_u8 data);
extern int valid_address(uaecptr adr, uae_u32 size);
#define get_real_address(ADR) (address_to_ptr(ADR))


/*========================*/
/* Logging                */
/*========================*/

#define write_log(STR) (addlog(STR))
extern char warning_buffer[256];


/*===================*/
/* UAE malloc define */
/*===================*/

#define xmalloc malloc

/*==========*/
/* Prefetch */
/*==========*/

#ifdef PREFETCH
#define fill_prefetch_0() prefetch_fill(pc)
#else
#define fill_prefetch_0()
#endif

/*========================*/
/* We don't have truncate */
/*========================*/

#undef truncate
#define truncate(x,y) 0


#endif










